#ifndef _DISPLAY_C
#define _DISPLAY_C


#include <stdlib.h>
#include <math.h>
#ifndef outb
    #define    outb(addr, data)    addr = (data)
#endif
#ifndef inb
    #define    inb(addr)            (addr)
#endif
#ifndef BV
    #define BV(bit)            (1<<(bit))
#endif
#ifndef cbi
    #define cbi(reg,bit)    reg &= ~(BV(bit))
#endif
#ifndef sbi
    #define sbi(reg,bit)    reg |= (BV(bit))
#endif
#ifndef cli
    #define cli()            #asm cli #endasm
#endif
#ifndef sei
    #define sei            #asm sei #endasm
#endif




#define FALSE	0
#define TRUE	-1




void InitHW(void)
{

    cbi(GLCD_CTRL_PORT, GLCD_CTRL_RS);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS0);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS1);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS2);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS3);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_RESET);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_RS);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_RW);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_E);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_CS0);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_CS1);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_CS2);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_CS3);
    sbi(GLCD_CTRL_DDR, GLCD_CTRL_RESET);
    outb(GLCD_DATA_PORT, 0x00);
	outb(GLCD_DATA_DDR, 0xFF);

}

void ControllerSelect(unsigned char controller)
{

    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS0);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS1);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS2);
    cbi(GLCD_CTRL_PORT, GLCD_CTRL_CS3);


    switch(controller)
    {
    case 0:    sbi(GLCD_CTRL_PORT, GLCD_CTRL_CS0); break;
    case 1:    sbi(GLCD_CTRL_PORT, GLCD_CTRL_CS1); break;
	case 2:	sbi(GLCD_CTRL_PORT, GLCD_CTRL_CS2); break;
	case 3:	sbi(GLCD_CTRL_PORT, GLCD_CTRL_CS3); break;
	default: break;
	}

}

void BusyWait(unsigned char controller)
{	
    #asm 
	cli 
	#endasm

	ControllerSelect(controller);
	outb(GLCD_DATA_PORT, 0xFF);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RS);
	outb(GLCD_DATA_DDR, 0x00);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	#asm 
    nop 
    nop
    #endasm
	while(inb(GLCD_DATA_PIN) & GLCD_STATUS_BUSY)
	{
		cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
		#ifndef Speed_mode
		#asm 
        nop 
        nop
        #endasm
		#endif
		#asm 
        nop 
        nop
        #endasm
		sbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
		#ifndef Speed_mode
		#asm 
        nop 
        nop
        #endasm
		#endif
		#asm 
        nop 
        nop
        #endasm

	}
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	outb(GLCD_DATA_DDR, 0xFF);
	 #asm 
     sei 
     #endasm
}


void ControlWrite(unsigned char controller, unsigned char data)
{

	#asm 
    cli 
    #endasm
	BusyWait(controller);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RS);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	outb(GLCD_DATA_DDR, 0xFF);
	outb(GLCD_DATA_PORT, data);
	#ifndef Speed_mode
	#asm
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    #endasm
	#endif
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	#asm
    sei
    #endasm
}

unsigned char ControlRead(unsigned char controller)
{
	register unsigned char data;

	#asm
    cli
    #endasm
	BusyWait(controller);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RS);
	outb(GLCD_DATA_DDR, 0x00);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	#ifndef Speed_mode
	#asm
    nop
    nop
    nop
    nop
    nop
    nop
    #endasm
	#endif
	#asm
    nop
    nop
    #endasm
	data = inb(GLCD_DATA_PIN);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	outb(GLCD_DATA_DDR, 0xFF);
	#asm
    sei
    #endasm


	return data;
}

void Write(unsigned char data)
{
	register unsigned char controller = (GrLcdState.lcdXAddr/GLCD_CONTROLLER_XPIXELS);

	#asm
    cli
    #endasm
	BusyWait(controller);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_RS);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	outb(GLCD_DATA_DDR, 0xFF);
	outb(GLCD_DATA_PORT, data);
	#ifndef Speed_mode
	#asm
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    #endasm
	#endif
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	#asm
    sei
    #endasm

	
	GrLcdState.ctrlr[controller].xAddr++;
	GrLcdState.lcdXAddr++;
	if(GrLcdState.lcdXAddr >= GLCD_XPIXELS)
	{
		GrLcdState.lcdYAddr++;
		SetYAddress(GrLcdState.lcdYAddr);
		SetXAddress(0);
	}
}

void SetXAddress(unsigned char xAddr)
{
	unsigned char i;
	
	GrLcdState.lcdXAddr = xAddr;

	
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		ControlWrite(i, GLCD_SET_Y_ADDR | 0x00);
		GrLcdState.ctrlr[i].xAddr = 0;
	}

	
	ControlWrite((GrLcdState.lcdXAddr/GLCD_CONTROLLER_XPIXELS),
		GLCD_SET_Y_ADDR | (GrLcdState.lcdXAddr & 0x3F));
}

void SetYAddress(unsigned char yAddr)
{
	unsigned char i;
	
	GrLcdState.lcdYAddr = yAddr;
	
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		ControlWrite(i, GLCD_SET_PAGE | yAddr);
	}
}

unsigned char Read(void)
{
	register unsigned char data;
	register unsigned char controller = (GrLcdState.lcdXAddr/GLCD_CONTROLLER_XPIXELS);

	#asm
    cli
    #endasm
	BusyWait(controller);	
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_RS);
	outb(GLCD_DATA_DDR, 0x00);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	#ifndef Speed_mode
	#asm
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    nop
    #endasm
	#endif
	data = inb(GLCD_DATA_PIN);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	
    
	BusyWait(controller);		
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_RS);
	outb(GLCD_DATA_DDR, 0x00);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	sbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	#ifndef Speed_mode
	#asm
    nop
    nop
    nop
    nop
    nop
    nop
    #endasm
	#endif
	#asm
    nop
    nop
    #endasm

	data = inb(GLCD_DATA_PIN);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_E);
	cbi(GLCD_CTRL_PORT, GLCD_CTRL_RW);
	#asm
    sei
    #endasm

	
	GrLcdState.ctrlr[controller].xAddr++;
	GrLcdState.lcdXAddr++;
	if(GrLcdState.lcdXAddr >= GLCD_XPIXELS)
	{
		GrLcdState.lcdYAddr++;
		SetYAddress(GrLcdState.lcdYAddr);
		SetXAddress(0);
	}
	return data;
}


void Reset(unsigned char resetState)
{
	

	if(resetState)
		cbi(GLCD_CTRL_PORT, GLCD_CTRL_RESET);
	else
		sbi(GLCD_CTRL_PORT, GLCD_CTRL_RESET);

}

void Init()
{
	unsigned char i;
    
	
	InitHW();
    
	
	Reset(FALSE);
	
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		ControlWrite(i, GLCD_ON_CTRL | GLCD_ON_DISPLAY);
	}
	
	ClearScreen();
	
	Home();
}

void Home(void)
{
	unsigned char i;
	
	StartLine(0);
	SetAddress(0,0);
	
	for(i=0; i<GLCD_NUM_CONTROLLERS; i++)
	{
		GrLcdState.ctrlr[i].xAddr = 0;
		GrLcdState.ctrlr[i].yAddr = 0;
	}
}

void ClearScreen(void)
{
	unsigned char pageAddr;
	unsigned char xAddr;

	
	for(pageAddr=0; pageAddr<(GLCD_YPIXELS>>3); pageAddr++)
	{
		
		SetAddress(0, pageAddr);
		
		for(xAddr=0; xAddr<GLCD_XPIXELS; xAddr++)
		{
			Write(0x00);
		}
	}
}


void StartLine(unsigned char start)
{
	ControlWrite(0, GLCD_START_LINE | start);
	ControlWrite(1, GLCD_START_LINE | start);
}

void SetAddress(unsigned char x, unsigned char yLine)
{
	
	SetYAddress(yLine);
	SetXAddress(x);
}






#endif
