#ifndef _DISPLAY_H
#define _DISPLAY_H

        #define Speed_mode

        #define GLCD_CTRL_PORT    PORTD    // PORT for LCD control signals
        #define GLCD_CTRL_DDR    DDRD    // DDR register of LCD_CTRL_PORT
        #define GLCD_CTRL_RS    0        // pin for LCD Register Select
        #define GLCD_CTRL_RW    1        // pin for LCD Read/Write
        #define GLCD_CTRL_E      2        // pin for LCD Enable
        #define GLCD_CTRL_CS0    3        // pin for LCD Controller 0 Chip Select
        #define GLCD_CTRL_CS1    4        // pin for LCD Controller 1 Chip Select(*)
        #define GLCD_CTRL_CS2    5        // pin for LCD Controller 2 Chip Select(*)
        #define GLCD_CTRL_CS3    6        // pin for LCD Controller 3 Chip Select(*)
        #define GLCD_CTRL_RESET    7        // pin for LCD Reset
        // (*) NOTE: additonal controller chip selects are optional and
        // will be automatically used per each step in 64 pixels of display size
        // Example: Display with 128 hozizontal pixels uses 2 controllers

        #define GLCD_DATA_PORT    PORTC    // PORT for LCD data signals
        #define GLCD_DATA_DDR    DDRC    // DDR register of LCD_DATA_PORT
        #define GLCD_DATA_PIN    PINC    // PIN register of LCD_DATA_PORT



        // absolute address of LCD Controller #0 CTRL and DATA registers
        #define GLCD_CONTROLLER0_CTRL_ADDR    0x1000
        #define GLCD_CONTROLLER0_DATA_ADDR    0x1001
        // offset of other controllers with respect to controller0
        #define GLCD_CONTROLLER_ADDR_OFFSET    0x0002



// LCD geometry defines (change these definitions to adapt code/settings)
#define GLCD_XPIXELS            128        // pixel width of entire display
#define GLCD_YPIXELS            64        // pixel height of entire display
#define GLCD_CONTROLLER_XPIXELS    64        // pixel width of one display controller


// HD61202/KS0108 command set
#define GLCD_ON_CTRL        0x3E    // 0011111X: lcd on/off control
#define GLCD_ON_DISPLAY        0x01    //        DB0: turn display on

#define GLCD_START_LINE        0xC0    // 11XXXXXX: set lcd start line

#define GLCD_SET_PAGE        0xB8    // 10111XXX: set lcd page (X) address
#define GLCD_SET_Y_ADDR        0x40    // 01YYYYYY: set lcd Y address

#define GLCD_STATUS_BUSY    0x80    // (1)->LCD IS BUSY
#define GLCD_STATUS_ONOFF    0x20    // (0)->LCD IS ON
#define GLCD_STATUS_RESET    0x10    // (1)->LCD IS RESET

// determine the number of controllers
// (make sure we round up for partial use of more than one controller)
#define GLCD_NUM_CONTROLLERS    ((GLCD_XPIXELS+GLCD_CONTROLLER_XPIXELS-1)/GLCD_CONTROLLER_XPIXELS)



typedef struct struct_GrLcdCtrlrStateType
{
	unsigned char xAddr;
	unsigned char yAddr;
} GrLcdCtrlrStateType;

typedef struct struct_GrLcdStateType
{
	unsigned char lcdXAddr;
	unsigned char lcdYAddr;
	GrLcdCtrlrStateType ctrlr[GLCD_NUM_CONTROLLERS];
} GrLcdStateType;



static GrLcdStateType GrLcdState;




    void InitHW(void);
    void ControllerSelect(unsigned char controller);
    void ControlWrite(unsigned char controller, unsigned char data);
    unsigned char  ControlRead(unsigned char controller);
    void SetXAddress(unsigned char xAddr);
    void SetYAddress(unsigned char yAddr);
    
    void BusyWait(unsigned char controller);
    void Reset(unsigned char resetState);
    void Write(unsigned char data);
    
    unsigned char  Read(void);
   
    void Init(void);
    void ClearScreen(void);
    void Home(void);
    void SetAddress(unsigned char x, unsigned char yLine);
    void StartLine(unsigned char start);
   #include "display.c"
#endif